<?php

/*
 * Copyright (C) 2025 Deciso B.V.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

namespace OPNsense\Firewall\DynamicAliases;

use OPNsense\Core\Config;

class CaptivePortalAliases
{
    public function collect()
    {
        $result = [];
        $cp = new \OPNsense\CaptivePortal\CaptivePortal();
        if ($cp->isEnabled()) {
            foreach ($cp->zones->zone->iterateItems() as $zone) {
                if (empty((string)$zone->enabled)) {
                    continue;
                }

                $zoneid = (string)$zone->zoneid;
                $result["__captiveportal_zone_{$zoneid}"] = [
                    "enabled" => "1",
                    "name" => "__captiveportal_zone_{$zoneid}",
                    "type" => "internal",
                    "description" => sprintf("%s %s", (string)$zone->descr, gettext("captiveportal")),
                    "content" => "",
                ];
            }
        }

        return $result;
    }
}
